﻿using mmd2timeline.Player.Model;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;

namespace mmd2timeline.Player.Controllers
{
    /// <summary>
    /// 镜头动作管理器
    /// 
    /// Warning:::
    /// 这部分参考了Seachlight的处理逻辑。
    /// 可能有版权问题，看是否需要通过调用seachlight插件的模式实现
    /// </summary>
    internal partial class CameraMotionController
    {
        #region 聚焦相关的代码

        /// <summary>
        /// 是否聚焦在原子上
        /// </summary>
        private bool _FocusOnAtom = false;

        /// <summary>
        /// 指示是否聚焦到原子
        /// </summary>
        public JSONStorableBool FocusOnAtomJSON;
        /// <summary>
        /// 聚焦原子选择器
        /// </summary>
        internal JSONStorableStringChooser FocusAtomJSON;
        /// <summary>
        /// 聚焦目标选择器
        /// </summary>
        internal JSONStorableStringChooser FocusReceiverJSON;

        /// <summary>
        /// 接收原子
        /// </summary>
        protected Atom _FocusAtom;
        /// <summary>
        /// 目标之前的位置
        /// </summary>
        Vector3 targetPastPosition = Vector3.zero;
        /// <summary>
        /// 目标位置
        /// </summary>
        Vector3 targetPosition;
        Quaternion pastRotation;

        /// <summary>
        /// 创建聚焦原子列表
        /// </summary>
        internal void MakeFocusAtomList()
        {
            List<string> targetChoices = new List<string>();
            foreach (string atomUID in SuperController.singleton.GetAtomUIDs())
            {
                var targetAtom = SuperController.singleton.GetAtomByUid(atomUID);
                if (targetAtom.hidden == false && targetAtom.GetBoolJSONParam("on").val == true && targetAtom.name != "CoreControl")
                {
                    targetChoices.Add(atomUID);
                }
            }
            FocusAtomJSON = new JSONStorableStringChooser(Lang.Get("Target Atom"), targetChoices, "None", Lang.Get("Target Atom"), SyncAtomChoices);
        }

        /// <summary>
        /// 同步原子选项
        /// </summary>
        /// <param name="atomUID"></param>
        protected void SyncAtomChoices(string atomUID)
        {
            List<string> receiverChoices = new List<string>();
            if (atomUID != null)
            {
                _FocusAtom = SuperController.singleton.GetAtomByUid(atomUID);
                if (_FocusAtom != null)
                {
                    FreeControllerV3[] controls = _FocusAtom.freeControllers;
                    foreach (var control in controls)
                    {
                        receiverChoices.Add(control.name);
                    }
                }
            }
            else
            {
                _FocusAtom = null;
            }
            FocusReceiverJSON.choices = receiverChoices;
            FocusReceiverJSON.val = "None";
        }

        /// <summary>
        /// 聚焦
        /// </summary>
        void FocusOn(string tagetId = null)
        {
            if (_FocusAtom == null)
            {
                _FocusAtom = SuperController.singleton.GetAtomByUid(FocusAtomJSON.val);
            }

            if (_FocusAtom != null)
            {
                if (string.IsNullOrEmpty(tagetId))
                {
                    tagetId = FocusReceiverJSON.val;
                }

                var target = _FocusAtom.freeControllers.FirstOrDefault(c => c.name == tagetId);//.GetStorableByID(FocusReceiverJSON.val).transform.position;

                SuperController.singleton.FocusOnController(target, alignUpDown: false);
            }
        }

        Vector3 GetFocusPosition(string tagetId = null)
        {
            if (_FocusAtom != null)
            {
                if (string.IsNullOrEmpty(tagetId))
                {
                    tagetId = FocusReceiverJSON.val;
                }

                return _FocusAtom.GetStorableByID(tagetId).transform.position;
            }

            return Vector3.zero;
        }

        FreeControllerV3 GetFocusController()
        {
            return _FocusAtom.freeControllers.FirstOrDefault(c => c.name == FocusReceiverJSON.val);
        }

        /// <summary>
        /// 获取聚焦角度
        /// </summary>
        /// <returns></returns>
        Quaternion GetFocusOnRotation(Vector3 position, Vector3 up, string tagetId = null)
        {
            if (_FocusAtom != null)
            {
                var pastDZx = 1.0f;
                var pastDZy = 1.0f;
                var pastDZz = 1.0f;

                var targetAtomPosition = GetFocusPosition(tagetId);

                Vector3 adjPosition;
                adjPosition.x = 0f;
                adjPosition.y = 0f;
                adjPosition.z = 0f;

                var currentAtomPosition = position;
                Vector3 adjcurrentPosition;
                adjcurrentPosition.x = 0f;
                adjcurrentPosition.y = 0f;
                adjcurrentPosition.z = 0f;

                Vector3 targetDiff = targetAtomPosition - targetPastPosition;
                if (Mathf.Abs(targetDiff.x) >= 0f)
                {
                    targetPosition.x = targetPastPosition.x + targetDiff.x * pastDZx;
                }
                if (Mathf.Abs(targetDiff.y) >= 0f)
                {
                    targetPosition.y = targetPastPosition.y + targetDiff.y * pastDZy;
                }
                if (Mathf.Abs(targetDiff.z) >= 0f)
                {
                    targetPosition.z = targetPastPosition.z + targetDiff.z * pastDZz;
                }

                targetPastPosition = targetPosition;

                var aim = (targetPosition + adjPosition) - (currentAtomPosition + adjcurrentPosition);

                pastRotation = Quaternion.LookRotation(aim, up);

                return pastRotation;
            }
            else
            {
                return Quaternion.identity;
            }
        }
        #endregion
    }
}
