﻿using Leap.Unity.Infix;
using MeshVR;
using mmd2timeline.Player.Model;
using System;
using UnityEngine;

namespace mmd2timeline.Player.Controllers
{
    /// <summary>
    /// 镜头动作控制，跟随。
    /// 
    /// Warning:::
    /// 这部分代码参考了Passenger的部分实现代码，不知是否存在版权问题
    /// </summary>
    internal partial class CameraMotionController
    {
        #region 初始参数设定
        /// <summary>
        /// 启用
        /// </summary>
        private bool _isActive = false;
        ///// <summary>
        ///// 是否启用FOV
        ///// </summary>
        //private bool _FOVEnabled = true;

        ///// <summary>
        ///// 是否聚焦在原子上
        ///// </summary>
        //private bool _FocusOnAtom = false;

        ///// <summary>
        ///// 允许在VR中使用FOV
        ///// </summary>
        //private bool _EnableFOVInVR = false;
        private bool _rotationLock = true;
        private bool _positionLock = true;

        //private bool _rotationLockNoRoll = false;
        /// <summary>
        /// 旋转平滑
        /// </summary>
        //private float _rotationSmoothing = 0.05f;//0-1
        private float _rotationOffsetX = 0f;
        private float _rotationOffsetY = 0f;
        private float _rotationOffsetZ = 0f;
        /// <summary>
        /// 位置平滑
        /// </summary>
        //private float _positionSmoothing = 0.05f;//0-1
        private float _positionOffsetX = 0f;
        private float _positionOffsetY = -0.1f;
        private float _positionOffsetZ = 0f;

        float _cameraScale = 0f;

        //private JSONStorableStringChooser _linkJSON;
        //private JSONStorableStringChooser _followerJSON;
        /// <summary>
        /// 是否启用全局缩放
        /// </summary>
        private bool _worldScaleEnabled = false;
        /// <summary>
        /// 全局缩放比例
        /// </summary>
        private float _worldScale = 1f;
        private float _eyesToHeadDistance = 0.0f;

        /// <summary>
        /// 之前的世界尺度
        /// </summary>
        private float _previousWorldScale;

        /// <summary>
        /// 占有者
        /// </summary>
        private Possessor _possessor;
        /// <summary>
        /// 开始方向偏移
        /// </summary>
        private Quaternion _startRotationOffset;
        /// <summary>
        /// 当前方向速度
        /// </summary>
        private Quaternion _currentRotationVelocity;
        /// <summary>
        /// 当前位置速度
        /// </summary>
        private Vector3 _currentPositionVelocity;

        #endregion

        #region UI参数
        //private JSONStorableStringChooser _toggleKeyJSON;
        //private KeyCode _toggleKey = KeyCode.BackQuote;

        public JSONStorableBool EnablePositionJSON;
        public JSONStorableBool EnableRotationJSON;
        //public JSONStorableBool FOVEnabledJSON;

        //public JSONStorableBool CameraEnabledJSON;

        //public JSONStorableBool DeactiveCameraWhenMainHUDOpenedJSON;

        //public JSONStorableFloat PositionSmoothingJSON;

        public JSONStorableFloat PositionOffsetXJSON;

        public JSONStorableFloat PositionOffsetYJSON;

        public JSONStorableFloat PositionOffsetZJSON;

        //public JSONStorableFloat RotationSmoothingJSON;

        public JSONStorableFloat RotationOffsetXJSON;

        public JSONStorableFloat RotationOffsetYJSON;

        public JSONStorableFloat RotationOffsetZJSON;

        //public JSONStorableBool EnabledFOVInVRJSON;

        public JSONStorableFloat CameraScaleJSON;

        //public JSONStorableBool FocusOnAtomJSON;
        #endregion

        /// <summary>
        /// 导航器快照，保存当前镜头状态
        /// </summary>
        private NavigationRigSnapshot _NavigationRigSnapshot;

        /// <summary>
        /// 是否已经启用
        /// </summary>
        private bool _isActived;

        /// <summary>
        /// 是否启用
        /// </summary>
        public bool IsActive
        {
            get
            {
                return _isActive;
            }
            set
            {
                if (_isActive != value)
                {
                    _isActive = value;

                    if (value)
                        Activate();
                    else
                        Deactivate();
                }
            }
        }

        /// <summary>
        /// 设置WindowCamera的开关状态
        /// </summary>
        /// <param name="on"></param>
        private void SetCameraOn(bool on = true)
        {
            if (WindowCamera == null)
            {
                return;
            }

            // 如果在监控模式，切换窗口
            if (on == SuperController.singleton.IsMonitorRigActive)
            {
                SuperController.singleton.ToggleMainMonitor();
            }

            var cameraOn = _CameraControl.GetBoolParamValue("cameraOn");

            if (cameraOn == on)
            {
                return;
            }

            WindowCamera.GetBoolJSONParam("on").val = on;
            _CameraControl.SetBoolParamValue("cameraOn", on);

            var displayControl = WindowCamera.GetStorableByID("DisplayControl") as FreeControllerV3;
            displayControl.on = !on;
        }

        /// <summary>
        /// 初始化
        /// </summary>
        private void InitFollowUI()
        {
            try
            {
                CameraScaleJSON = new JSONStorableFloat(Lang.Get("Camera Scale"), _cameraScale, new JSONStorableFloat.SetFloatCallback(v => _cameraScale = v), -1f, 1f, true, true);

                EnablePositionJSON = new JSONStorableBool(Lang.Get("Enabled Position Control"), true, new JSONStorableBool.SetBoolCallback(v => _positionLock = v));
                EnableRotationJSON = new JSONStorableBool(Lang.Get("Enabled Rotation Control"), true, new JSONStorableBool.SetBoolCallback(v => _rotationLock = v));

                FocusOnAtomJSON = new JSONStorableBool(Lang.Get("Focus On ..."), _FocusOnAtom);
                PositionOffsetXJSON = new JSONStorableFloat(Lang.Get("Position X"), _positionOffsetX, new JSONStorableFloat.SetFloatCallback(v => _positionOffsetX = v), -2f, 2f, false, true);
                PositionOffsetYJSON = new JSONStorableFloat(Lang.Get("Position Y"), _positionOffsetY, new JSONStorableFloat.SetFloatCallback(v => _positionOffsetY = v), -2f, 2f, false, true);
                PositionOffsetZJSON = new JSONStorableFloat(Lang.Get("Position Z"), _positionOffsetZ, new JSONStorableFloat.SetFloatCallback(v => _positionOffsetZ = v), -2f, 2f, false, true);
                RotationOffsetXJSON = new JSONStorableFloat(Lang.Get("Rotation X"), _rotationOffsetX, new JSONStorableFloat.SetFloatCallback(v => _rotationOffsetX = v), -180, 180, true, true);
                RotationOffsetYJSON = new JSONStorableFloat(Lang.Get("Rotation Y"), _rotationOffsetY, new JSONStorableFloat.SetFloatCallback(v => _rotationOffsetY = v), -180, 180, true, true);
                RotationOffsetZJSON = new JSONStorableFloat(Lang.Get("Rotation Z"), _rotationOffsetZ, new JSONStorableFloat.SetFloatCallback(v => _rotationOffsetZ = v), -180, 180, true, true);

                MakeFocusAtomList();

                FocusReceiverJSON = new JSONStorableStringChooser(Lang.Get("Target Receiver"), null, "None", Lang.Get("Target Receiver"));

                // 获取拥有者
                _possessor = SuperController.singleton.centerCameraTarget.transform.GetComponent<Possessor>();

                // 广播OnActionsProviderAvailable
                SuperController.singleton.BroadcastMessage("OnActionsProviderAvailable", this, SendMessageOptions.DontRequireReceiver);
            }
            catch (Exception e)
            {
                LogUtil.Debug(e, "Failed to initialize plugin: ");
            }
        }

        /// <summary>
        /// 更新镜头
        /// </summary>
        /// <param name="position"></param>
        /// <param name="rotation"></param>
        /// <param name="fov"></param>
        /// <param name="orthographic"></param>
        public void UpdateCamera(Vector3 position, Quaternion rotation, float fov, bool orthographic, string tagetId = null)
        {
            try
            {
                // 如果不启用，直接返回
                if (!_isActive)
                {
                    Deactivate();
                    return;
                }

                // 如果没有通过健康检查，则返回
                if (!HealthCheck()) return;

                // 获取导航装置
                var navigationRig = SuperController.singleton.navigationRig;

                if (config.UseWindowCamera && WindowCamera != null)
                {
                    var navigationRigPosition = GetPosition(position, rotation, _CameraTransform, isCamera: true);

                    if (_positionLock)
                    {
                        _CameraTransform.position = navigationRigPosition;
                    }

                    if (_rotationLock)
                    {
                        var navigationRigRotation = GetRotation(navigationRigPosition, rotation, _CameraTransform, tagetId);
                        _CameraTransform.rotation = navigationRigRotation;
                    }

                    _CameraControl.cameraFOV = fov;
                    _CameraControl.cameraToControl.orthographic = orthographic;
                }
                else
                {
                    var navigationRigParent = SuperController.singleton.navigationRigParent;
                    var monitorCamera = SuperController.singleton.MonitorCenterCamera;

                    var navigationRigPosition = GetPosition(position, rotation, navigationRig);

                    if (_positionLock)
                    {
                        navigationRig.position = navigationRigPosition;
                    }

                    if (_rotationLock)
                    {
                        var navigationRigRotation = GetRotation(position, rotation, navigationRig, tagetId);
                        navigationRig.rotation = navigationRigRotation;
                    }

                    SuperController.singleton.monitorCameraFOV = fov;
                    monitorCamera.orthographic = orthographic;
                }
            }
            catch (Exception e)
            {
                LogUtil.Debug(e, "Failed to update: ");
            }
        }

        /// <summary>
        /// 获取旋转数据
        /// </summary>
        /// <param name="navigationRig">导航装置</param>
        /// <param name="rotationSmoothing">旋转平滑</param>
        private Quaternion GetRotation(Vector3 position, Quaternion rotation, Transform navigationRig, string tagetId = null)
        {
            // 获取导航装置的旋转
            var navigationRigRotation = rotation;

            // 如果选中聚焦到原子
            if (FocusOnAtomJSON.val || !string.IsNullOrEmpty(tagetId))
            {
                navigationRigRotation = GetFocusOnRotation(position, navigationRigRotation.GetUp(), tagetId);
            }
            else if (rotation == Quaternion.identity)
            {
                navigationRigRotation = navigationRig.rotation;
            }

            // TODO? Necessary?
            if (_startRotationOffset == Quaternion.identity)
                navigationRigRotation *= _startRotationOffset;

            navigationRigRotation *= Quaternion.Euler(_rotationOffsetX, _rotationOffsetX, _rotationOffsetX);

            if (config.CameraRotationSmoothing > 0)
            {
                navigationRigRotation = SmoothDamp(navigationRig.rotation, navigationRigRotation, ref _currentRotationVelocity, config.CameraRotationSmoothing);
            }

            return navigationRigRotation;
        }

        /// <summary>
        /// 获取位置数据
        /// </summary>
        /// <param name="navigationRig">导航装置</param>
        /// <param name="positionSmoothing">位置平滑</param>
        private Vector3 GetPosition(Vector3 position, Quaternion rotation, Transform navigationRig, bool isCamera = false)
        {
            // 结果位置
            Vector3 resultPosition;
            if (!isCamera)
            {
                // 镜头三角
                var cameraDelta = CameraTarget.centerTarget.transform.position - navigationRig.transform.position - CameraTarget.centerTarget.transform.rotation * new Vector3(0, 0, _eyesToHeadDistance);
                resultPosition = position - cameraDelta + rotation * new Vector3(_positionOffsetX, _positionOffsetY, _positionOffsetZ);
            }
            else
            {
                resultPosition = position + rotation * new Vector3(_positionOffsetX, _positionOffsetY, _positionOffsetZ);
            }

            // 位置平滑大于0
            if (config.CameraPositionSmoothing > 0)
                resultPosition = Vector3.SmoothDamp(navigationRig.position, resultPosition, ref _currentPositionVelocity, config.CameraPositionSmoothing, Mathf.Infinity, Time.smoothDeltaTime);

            // 更新导航仪的位置
            return resultPosition;
        }

        /// <summary>
        /// 更新
        /// </summary>
        public void Update(Vector3 position, float fov, Quaternion rotation, string tagetId = null)
        {
            UpdateCamera(position, rotation, fov, false, tagetId);
        }

        /// <summary>
        /// 初始化镜头跟随的设置
        /// </summary>
        /// <param name="settings"></param>
        internal void InitFollowSettings(CameraSetting settings)
        {
            this.PositionOffsetXJSON.val = settings.PositionOffsetX;
            this.PositionOffsetYJSON.val = settings.PositionOffsetY;
            this.PositionOffsetZJSON.val = settings.PositionOffsetZ;
            this.RotationOffsetXJSON.val = settings.RotationOffsetX;
            this.RotationOffsetYJSON.val = settings.RotationOffsetY;
            this.RotationOffsetZJSON.val = settings.RotationOffsetZ;
        }

        /// <summary>
        /// 禁用导航
        /// </summary>
        /// <param name="disable"></param>
        internal void DisableNavigation(bool disable = true)
        {
            LogUtil.Debug($"DisableNavigation::{disable}");

            // 未启用镜头，跳过处理
            if (!config.EnableCamera)
                return;

            if (config.UseWindowCamera)
            {
                SetCameraOn(disable);
            }
            else
            {
                // 如果状态一致跳过处理
                if (SuperController.singleton.navigationDisabled == disable)
                    return;

                // 如果是要禁用导航，则重置导航装置的位置和角度
                if (disable)
                {
                    SuperController.singleton.ResetNavigationRigPositionRotation();
                }

                // 只有镜头处于启用状态才能禁用导航
                if (disable && !_isActive) { return; }

                if (GlobalSceneOptions.singleton != null)
                {
                    GlobalSceneOptions.singleton.disableNavigation = disable;
                }
                SuperController.singleton.disableNavigation = disable;
            }
        }

        /// <summary>
        /// 激活
        /// </summary>
        private void Activate()
        {
            // 如果已经时激活状态，返回
            if (_isActived) return;
            // 进行健康度检查，如果检查不通过，返回
            if (!HealthCheck()) return;

            _NavigationRigSnapshot = NavigationRigSnapshot.Snap();

            DisableNavigation();

            // 超级控制器
            var superController = SuperController.singleton;
            // 导航装置
            var navigationRig = superController.navigationRig;

            // 监控设备没有被激活
            var offsetStartRotation = !superController.MonitorRig.gameObject.activeSelf;
            // 设定开始旋转的偏移量
            if (offsetStartRotation)
                _startRotationOffset = Quaternion.Euler(0, navigationRig.eulerAngles.y - _possessor.transform.eulerAngles.y, 0f);

            // 应用全局缩放
            ApplyWorldScale();

            // 设定为激活状态
            _isActived = true;
        }

        /// <summary>
        /// 应用全局缩放
        /// </summary>
        private void ApplyWorldScale()
        {
            // 如果没有开启全局缩放，则直接返回
            if (!_worldScaleEnabled) return;

            // 前一个全局缩放尺寸
            _previousWorldScale = SuperController.singleton.worldScale;
            // 设定全局缩放尺寸
            SuperController.singleton.worldScale = _worldScale;
        }

        /// <summary>
        /// 取消激活
        /// </summary>
        private void Deactivate()
        {
            if (!_isActived) return;

            _NavigationRigSnapshot?.Restore();

            DisableNavigation(false);

            if (_previousWorldScale > 0f)
            {
                SuperController.singleton.worldScale = _previousWorldScale;
                _previousWorldScale = 0f;
            }

            _currentPositionVelocity = Vector3.zero;
            _currentRotationVelocity = Quaternion.identity;
            _startRotationOffset = Quaternion.identity;

            _isActived = false;
        }

        /// <summary>
        /// 重新应用
        /// </summary>
        private void Reapply()
        {
            if (!_isActived) return;

            Deactivate();
            Activate();
        }

        /// <summary>
        /// 健康度检查
        /// </summary>
        /// <returns></returns>
        private bool HealthCheck()
        {
            return true;
        }

        /// <summary>
        /// 终止启用
        /// </summary>
        /// <param name="message"></param>
        private void AbortActivation(string message)
        {
            LogUtil.Debug(message);
            this.IsActive = false;
            //Deactivate();
        }

        /// <summary>
        /// 销毁跟随对象
        /// </summary>
        private void DisposeFollow()
        {
            this.Deactivate();
        }

        // Source: https://gist.github.com/maxattack/4c7b4de00f5c1b95a33b
        /// <summary>
        /// 平滑湿润？
        /// </summary>
        /// <param name="current">当前</param>
        /// <param name="target">目标</param>
        /// <param name="currentVelocity">当前速度</param>
        /// <param name="smoothTime">平滑时间</param>
        /// <returns></returns>
        public static Quaternion SmoothDamp(Quaternion current, Quaternion target, ref Quaternion currentVelocity, float smoothTime)
        {
            // account for double-cover
            var Dot = Quaternion.Dot(current, target);
            var Multi = Dot > 0f ? 1f : -1f;
            target.x *= Multi;
            target.y *= Multi;
            target.z *= Multi;
            target.w *= Multi;
            // smooth damp (nlerp approx)
            var Result = new Vector4(
                Mathf.SmoothDamp(current.x, target.x, ref currentVelocity.x, smoothTime),
                Mathf.SmoothDamp(current.y, target.y, ref currentVelocity.y, smoothTime),
                Mathf.SmoothDamp(current.z, target.z, ref currentVelocity.z, smoothTime),
                Mathf.SmoothDamp(current.w, target.w, ref currentVelocity.w, smoothTime)
            ).normalized;
            // compute deriv
            var dtInv = 1f / Time.smoothDeltaTime;
            currentVelocity.x = (Result.x - current.x) * dtInv;
            currentVelocity.y = (Result.y - current.y) * dtInv;
            currentVelocity.z = (Result.z - current.z) * dtInv;
            currentVelocity.w = (Result.w - current.w) * dtInv;
            return new Quaternion(Result.x, Result.y, Result.z, Result.w);
        }
    }
}
