﻿using LibMMD.Motion;
using LibMMD.Unity3D;
using mmd2timeline;
using System;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;
using mmd2timeline.Player.Model;

namespace mmd2timeline.Player.Controllers
{
    /// <summary>
    /// 镜头动作控制器
    /// </summary>
    internal partial class CameraMotionController : DelayTimeController
    {
        /// <summary>
        /// 镜头选择器
        /// </summary>
        public JSONStorableStringChooser cameraChooser;

        private static CameraMotionController _instance;
        private static object _lock = new object();

        /// <summary>
        /// 镜头控制器的单例
        /// </summary>
        public static CameraMotionController GetInstance()
        {
            lock (_lock)
            {
                if (_instance == null)
                {
                    _instance = new CameraMotionController();
                }

                return _instance;
            }
        }

        /// <summary>
        /// 实例化MMDCameraAtom
        /// </summary>
        /// <param name="cameraAtom"></param>
        private CameraMotionController()
        {
            Init();
        }

        /// <summary>
        /// 初始化
        /// </summary>
        private void Init()
        {
            try
            {
                base.Init(Lang.Get("Camera Motion"));

                // 初始化采样率选择器
                this.sampleRateChooser = new JSONStorableStringChooser(Lang.Get("Sample Mode"), new List<string>
                {
                    Lang.Get("EveryFrame"),// 每一帧
                    Lang.Get("KeyFrame")// 关键帧
                }, Lang.Get("KeyFrame"), Lang.Get("Sample Mode"));

                // 创建MMD镜头对象，名称为MMDCameraObject
                //this._MmdCamera = MmdCameraObject.CreateGameObject("MMDCameraObject").GetComponent<MmdCameraObject>();

                GameObject root = new GameObject("mmd2timeline camera root");
                root.transform.localPosition = Vector3.zero;
                root.transform.localScale = new Vector3(0.1f, 0.1f, 0.1f);
                root.transform.localEulerAngles = new Vector3(0, 180, 0);

                _MmdCamera = MmdCameraObject.CreateGameObject();
                _MmdCamera.transform.SetParent(root.transform);
                _MmdCamera.transform.localPosition = Vector3.zero;
                _MmdCamera.transform.localScale = Vector3.one;
                _MmdCamera.transform.localRotation = Quaternion.identity;

                this._MmdCamera.CameraUpdated += MmdCamera_CameraUpdated;

                // 镜头选择器
                this.cameraChooser = new JSONStorableStringChooser(Lang.Get("Camera Motion"), noneStrings, noneStrings, "None", Lang.Get("Camera Motion"), new JSONStorableStringChooser.SetStringCallback((p) =>
                {
                    if (p == "None")
                    {
                        this.ImportVmd(null);
                    }
                    else
                    {
                        this.ImportVmd(p);
                    }
                }));

                InitFollowUI();
            }
            catch (Exception ex)
            {
                LogUtil.LogError(ex, $"CameraMotionController::Init:");
            }
        }

        /// <summary>
        /// 镜头更新事件处理方法
        /// </summary>
        /// <param name="position"></param>
        /// <param name="rotation"></param>
        /// <param name="fov"></param>
        /// <param name="orthographic"></param>
        private void MmdCamera_CameraUpdated(Vector3 position, Quaternion rotation, float fov, bool orthographic)
        {
            this.UpdateCamera(position, rotation, fov, orthographic);
        }

        /// <summary>
        /// 开始
        /// </summary>
        private void Start()
        {
        }

        /// <summary>
        /// 初始化播放参数
        /// </summary>
        /// <param name="choices"></param>
        /// <param name="displayChoices"></param>
        /// <param name="choice"></param>
        /// <param name="settings"></param>
        internal void InitPlay(List<string> choices, List<string> displayChoices, string choice, CameraSetting settings)
        {
            this.SetTimeDelay(settings.TimeDelay);
            this.cameraChooser.displayChoices = displayChoices;
            this.cameraChooser.choices = choices;
            this.cameraChooser.val = choice;

            InitFollowSettings(settings);
        }

        /// <summary>
        /// 清理播放数据
        /// </summary>
        public void Clear()
        {
            this.cameraChooser.choices = noneStrings;
            this.cameraChooser.displayChoices = noneStrings;
            this.cameraChooser.val = noneStrings.FirstOrDefault();
        }

        /// <summary>
        /// 进度变化的处理
        /// </summary>
        /// <param name="value"></param>
        protected override void OnProgressChange(float value)
        {
            base.OnProgressChange(value);

                            UnityEngine.Debug.Log("Camera OnProgressChange " + value);
            // 调用MMDCamera的播放进度
            this._MmdCamera.SetPlayPos((double)value/*, config.CameraOnlyKeyFrame*/);
        }

        /// <summary>
        /// 检查是否已经播放完毕
        /// </summary>
        /// <param name="value"></param>
        /// <returns></returns>
        protected override bool CheckEnd(float value)
        {
            return base.CheckEnd(value);
        }

        /// <summary>
        /// 导入VMD文件
        /// </summary>
        /// <param name="path"></param>
        /// <param name="audioSource"></param>
        private void ImportVmd(string path, string audioSource = "")
        {
            try
            {
                // 初始化MMD镜头的本地位置
                this._MmdCamera.transform.localPosition = Vector3.zero;
                // 初始化MMD镜头的本地缩放
                this._MmdCamera.transform.localScale = Vector3.one;
                // 初始化MMD镜头的本地旋转
                this._MmdCamera.transform.localRotation = Quaternion.identity;
                // 加载MMD镜头数据文件
                this._MmdCamera.LoadCameraMotion(path);

                this._CameraMotion = this._MmdCamera._cameraMotion;

                // 获取MMD镜头的最后一帧
                int key = 0;
                if (_CameraMotion != null && _CameraMotion.KeyFrames.Count > 0)
                {
                    key = _CameraMotion.KeyFrames.Max(k => k.Key);
                }

                MaxTime = (float)((double)key / 30.0);
                OnLengthGot(MaxTime);
            }
            catch (Exception ex)
            {
                LogUtil.LogError(ex);
            }
        }

        public override void OnDisable()
        {
            //throw new NotImplementedException();
        }

        public override void OnEnable()
        {
            //throw new NotImplementedException();
        }

        public override void OnDestroy()
        {
            this.Clear();

            DisposeFollow();

            this._MmdCamera = null;

            _instance = null;
            base.OnDestroy();
        }

        public MmdCameraObject MMDCamera
        {
            get
            {
                return _MmdCamera;
            }
        }


        /// <summary>
        /// 采样模式选择器
        /// </summary>
        public JSONStorableStringChooser sampleRateChooser;

        /// <summary>
        /// MMD镜头对象
        /// </summary>
        private MmdCameraObject _MmdCamera;

        /// <summary>
        /// 镜头数据对象
        /// </summary>
        CameraMotion _CameraMotion;
    }
}
