﻿using mmd2timeline.Player.Model;

namespace mmd2timeline.Player.Controllers
{
    /// <summary>
    /// 延时控制器
    /// </summary>
    internal class DelayTimeController : ProgressController
    {
        /// <summary>
        /// 时间延迟
        /// </summary>
        float delay = 0f;

        /// <summary>
        /// 开始时间
        /// </summary>
        public JSONStorableFloat timeDelay;

        /// <summary>
        /// 最大时间
        /// </summary>
        internal new float MaxTime
        {
            get
            {
                return base.MaxTime;
            }
            set
            {
                base.MaxTime = value;
                this.timeDelay.max = value;
                this.timeDelay.min = 0 - value;

                this.playProgress.max = value;
            }
        }

        /// <summary>
        /// 初始化函数
        /// </summary>
        /// <param name="controllerName"></param>
        protected new void Init(string controllerName)
        {
            base.Init(controllerName);

            // 初始化时间延迟，变动时调用SetDelayTime
            this.timeDelay = new JSONStorableFloat($"{controllerName} {Lang.Get("Delay")}", 0f, new JSONStorableFloat.SetFloatCallback(this.SetTimeDelay), 0f, 0f, false, true);
        }

        /// <summary>
        /// 初始化开始和结束时间
        /// </summary>
        /// <param name="delay"></param>
        internal void SetTimeDelay(float delay)
        {
            this.delay = delay;

            if (this.timeDelay.val != delay)
            {
                this.timeDelay.val = delay;
            }
        }

        /// <summary>
        /// 设置进度条
        /// </summary>
        /// <param name="value"></param>
        internal new void SetProgress(float value)
        {
            var progress = this.GetRealProgress(value);

            if (progress < 0)
            {
                progress = 0;
            }

            base.SetProgress(progress);
        }

        /// <summary>
        /// 检查是否进行到末尾
        /// </summary>
        /// <param name="value"></param>
        /// <returns></returns>
        protected override bool CheckEnd(float value)
        {
            return base.CheckEnd(value);
        }

        /// <summary>
        /// 获得真实的进度条进度
        /// </summary>
        /// <param name="value"></param>
        /// <returns></returns>
        protected new float GetRealProgress(float value)
        {
            return value - this.delay;
        }
    }
}
