﻿using mmd2timeline.Player.Model;

namespace mmd2timeline.Player.Controllers
{
    internal abstract class ProgressController : MMDBaseController
    {
        /// <summary>
        /// 当前播放进度
        /// </summary>
        public JSONStorableFloat playProgress;
        /// <summary>
        /// 进度变化的事件
        /// </summary>
        /// <param name="value"></param>
        public delegate void ProgressChanged(float value);

        /// <summary>
        /// 当进度变化时调用的方法
        /// </summary>
        public ProgressChanged OnProgressChanged;

        /// <summary>
        /// 播放结束的事件
        /// </summary>
        public delegate void OnEndCallBack();

        /// <summary>
        /// 当播放结束时调用的方法
        /// </summary>
        public OnEndCallBack OnEnd;

        /// <summary>
        /// 当加载完成后的回调函数
        /// </summary>
        /// <param name="maxTime"></param>
        public delegate void OnLoadedCallback(ProgressController sender, float maxTime);

        /// <summary>
        /// 加载完成
        /// </summary>
        public event OnLoadedCallback OnLoaded;

        /// <summary>
        /// 最大时间
        /// </summary>
        float maxTime = 0f;

        internal bool IsEnd = false;

        /// <summary>
        /// 最大时间
        /// </summary>
        internal float MaxTime
        {
            get
            {
                return maxTime;
            }
            set
            {
                maxTime = value;
            }
        }

        /// <summary>
        /// 播放进度
        /// </summary>
        internal float Progress
        {
            get
            {
                return this.playProgress.val;
            }
        }

        /// <summary>
        /// 初始化有进度的管理员对象
        /// </summary>
        internal ProgressController()
        {
        }

        /// <summary>
        /// 初始化函数
        /// </summary>
        /// <param name="managerName"></param>
        protected void Init(string managerName)
        {
            // 初始化播放进度条变量
            this.playProgress = new JSONStorableFloat($"{managerName} {Lang.Get("Progress")}", 0f, this.OnProgressChange, 0f, 0f, constrain: true, interactable: true);
        }

        /// <summary>
        /// 设置进度条
        /// </summary>
        /// <param name="value"></param>
        internal void SetProgress(float value)
        {
            this.playProgress.valNoCallback = value;
            OnProgressChange(value);
        }

        /// <summary>
        /// 当获取到长度后执行的方法
        /// </summary>
        /// <param name="maxTime"></param>
        protected void OnLengthGot(float maxTime)
        {
            OnLoaded?.Invoke(this, maxTime);
        }

        /// <summary>
        /// 获取相对进度
        /// </summary>
        /// <param name="value"></param>
        /// <returns></returns>
        protected float GetRealProgress(float value)
        {
            return value;
        }

        /// <summary>
        /// 检查一个值是否到达末尾
        /// </summary>
        /// <param name="value"></param>
        /// <returns></returns>
        protected virtual bool CheckEnd(float value)
        {
            return value + 0.1f >= this.MaxTime;
        }

        /// <summary>
        /// 当进度改变执行的方法
        /// </summary>
        /// <param name="value"></param>
        protected virtual void OnProgressChange(float value)
        {
            OnProgressChanged?.Invoke(value);
        }

        public override void OnDisable()
        {
        }

        public override void OnEnable()
        {
        }
    }
}
