using mmd2timeline.Player.Model;

namespace mmd2timeline.Player.Controllers
{
    internal abstract class StartAndEndTimeController : ProgressController
    {
        /// <summary>
        /// 开始时间
        /// </summary>
        float start = 0f;
        /// <summary>
        /// 结束时间
        /// </summary>
        float end = 0f;

        /// <summary>
        /// 开始时间
        /// </summary>
        public JSONStorableFloat startTime;

        /// <summary>
        /// 结束时间
        /// </summary>
        public JSONStorableFloat endTime;

        /// <summary>
        /// 最大时间
        /// </summary>
        internal new float MaxTime
        {
            get { return base.MaxTime; }
            set
            {
                base.MaxTime = value;
                this.startTime.max = value;
                this.endTime.max = value;

                this.end = (this.end > 0) ? this.end : base.MaxTime;

                this.playProgress.max = this.end;

                SetStartTime(this.start);
                SetEndTime(this.end);
            }
        }

        /// <summary>
        /// 初始化函数
        /// </summary>
        /// <param name="managerName"></param>
        protected new void Init(string managerName)
        {
            base.Init(managerName);

            // 初始化开始时间，变动时调用SetStartTime
            this.startTime = new JSONStorableFloat($"{managerName} {Lang.Get("Start Time")}", 0f, new JSONStorableFloat.SetFloatCallback(this.SetStartTime), 0f, 0f, true, true);
            // 初始化结束时间，变动时调用SetEndTime
            this.endTime = new JSONStorableFloat($"{managerName} {Lang.Get("End Time")}", 0f, new JSONStorableFloat.SetFloatCallback(this.SetEndTime), 0f, 0f, true, true);
        }

        /// <summary>
        /// 初始化开始和结束时间
        /// </summary>
        /// <param name="start"></param>
        /// <param name="end"></param>
        internal void InitStartAndEndTime(float start, float end)
        {
            this.start = (start > 0) ? start : 0;
            this.end = end > 0 ? end : 0;
        }

        /// <summary>
        /// 设置开始时间
        /// </summary>
        /// <param name="time"></param>
        protected virtual void SetStartTime(float time)
        {
            if (time > this.end)
            {
                time = this.end;
            }

            // 设置播放进度的最小时间不能小于开始时间
            this.playProgress.min = time;// + 0.001f;
            //if (time<0.001f)
            //{
            //    this.playProgress.min = 0.001f;//修改限制滚动播放最小值
            //}

            if (this.startTime.val != time)
            {
                this.startTime.val = time;
            }
        }

        /// <summary>
        /// 设置结束时间
        /// </summary>
        /// <param name="val"></param>
        protected virtual void SetEndTime(float time)
        {
            if (time < 0)
            {
                time = 0;
            }

            if (time < this.start)
            {
                time = this.start;
            }

            // 如果设定的结束时间大于最大时间，则更新最大时间
            if (time > MaxTime)
            {
                time = MaxTime;
            }

            // 设置播放进度的最大值为设定时间
            this.playProgress.max = time;// - 0.001f;

            // 如果结束时间与设定时间不一致，则更新结束时间
            if (this.endTime.val != time)
            {
                this.endTime.val = time;
            }
        }

        /// <summary>
        /// 修改结束检查方法，检查时判定结束时间
        /// </summary>
        /// <param name="value"></param>
        /// <returns></returns>
        protected override bool CheckEnd(float value)
        {
            return value + 0.1f >= this.endTime.val;
        }
    }
}
