﻿using System;
using UnityEngine;
using UnityEngine.UI;

namespace mmd2timeline.Player
{
    internal partial class Player
    {
        /// <summary>
        /// 状态文本
        /// </summary>
        public JSONStorableString statusText;

        private void InitStatusUI()
        {
            try
            {
                LogUtil.Debug($"InitStatusUI::Start");

                statusText = new JSONStorableString("StatusText", "MMD2Timeline");

                Canvas canvas = new GameObject("StatusCanvas").AddComponent<Canvas>();
                canvas.renderMode = RenderMode.ScreenSpaceOverlay;
                SuperController.singleton.AddCanvas(canvas);
                canvas.gameObject.AddComponent<GraphicRaycaster>();
                CanvasScaler canvasScaler = canvas.gameObject.AddComponent<CanvasScaler>();
                canvasScaler.scaleFactor = 1f;
                canvasScaler.dynamicPixelsPerUnit = 1f;
                VerticalLayoutGroup verticalLayoutGroup = canvas.gameObject.AddComponent<VerticalLayoutGroup>();
                verticalLayoutGroup.childAlignment = TextAnchor.UpperRight;
                verticalLayoutGroup.childControlWidth = false;
                verticalLayoutGroup.childForceExpandHeight = false;
                verticalLayoutGroup.childForceExpandWidth = false;
                verticalLayoutGroup.spacing = 10f;
                verticalLayoutGroup.padding.top = 20;
                verticalLayoutGroup.padding.right = 20;
                verticalLayoutGroup.GetComponent<RectTransform>().sizeDelta = new Vector2(400f, 400f);

                UIDynamicTextField component = UnityEngine.Object.Instantiate(manager.configurableTextFieldPrefab).GetComponent<UIDynamicTextField>();
                component.backgroundColor = Color.clear;
                component.textColor = Color.white;
                component.UItext.alignment = TextAnchor.UpperRight;
                component.UItext.supportRichText = true;
                statusText.dynamicText = component;
                component.height = 400f;
                component.GetComponent<RectTransform>().sizeDelta = new Vector2(400f, component.height);
                component.transform.SetParent(verticalLayoutGroup.transform, worldPositionStays: false);
            }
            catch (Exception e)
            {
                LogUtil.LogError(e, $"InitControlUI::");
            }
        }

        /// <summary>
        /// 显示状态消息
        /// </summary>
        /// <param name="msg"></param>
        /// <param name="fontSize"></param>
        /// <param name="anchor"></param>
        internal void ShowStatusMessage(string msg = null, int fontSize = 24, TextAnchor anchor = TextAnchor.UpperRight)
        {
            if (string.IsNullOrEmpty(msg))
            {
                msg = $"{PLUGIN_NAME} v{VERSION}";
            }

            statusText.val = msg;
            statusText.text.alignment = anchor;
            statusText.text.fontSize = fontSize;

            ShowUIElement(statusText, true);
        }

        /// <summary>
        /// 隐藏状态消息
        /// </summary>
        internal void HideStatusMessage()
        {
            ShowUIElement(statusText, false);
        }

        /// <summary>
        /// 销毁状态UI
        /// </summary>
        void DestroyStatusUI()
        {
            UnityEngine.Object.Destroy(GameObject.Find("StatusCanvas"));
        }
    }
}
