using MacGruber;
using mmd2timeline.Player.Controllers;
using mmd2timeline.Player.Model;
using System;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;
using UnityEngine.Events;
using UnityEngine.UI;

namespace mmd2timeline.Player
{
    internal partial class Player
    {
        /// <summary>
        /// 调试信息
        /// </summary>
        JSONStorableString _DebugInfo;
        /// <summary>
        /// 是否显示调试信息
        /// </summary>
        internal JSONStorableBool _ShowDebugInfoJSON;

        #region UI配置相关的方法

        /// <summary>
        /// 播放设置UI清单
        /// </summary>
        List<object> _PlayUIs = new List<object>();

        /// <summary>
        /// 编辑UI
        /// </summary>
        List<object> _EditUIs = new List<object>();

        /// <summary>
        /// 动作播放相关的UI
        /// </summary>
        List<object> _MMDUIs = new List<object>();

        /// <summary>
        /// 加载UI
        /// </summary>
        List<object> _LoadUIs = new List<object>();

        /// <summary>
        /// 导入文件和目录的按钮
        /// </summary>
        List<object> _LoadButtons = new List<object>();

        /// <summary>
        /// 调试UI
        /// </summary>
        GroupUI _DebugUIs;

        /// <summary>
        /// 镜头设置UI
        /// </summary>
        GroupUI _CameraSettingUIs;

        GroupUI heelSettingsGroup;

        /// <summary>
        /// 播放按钮
        /// </summary>
        UIDynamicButton _UIPlayButton;

        /// <summary>
        /// 播放列表选择器
        /// </summary>
        JSONStorableStringChooser _PlaylistChooser;

        /// <summary>
        /// 加载双按钮
        /// </summary>
        UIDynamicTwinButton _LoadTwinButton;

        /// <summary>
        /// 播放速度
        /// </summary>
        JSONStorableFloat _PlaySpeedJSON;

        /// <summary>
        /// 加载提示信息
        /// </summary>
        UIDynamicTextField _LoadTips;

        /// <summary>
        /// 插件信息
        /// </summary>
        UIDynamicTextField _PluginInfo;

        /// <summary>
        /// 收藏按钮标签
        /// </summary>
        Text _FavoriteLabel;

        /// <summary>
        /// 选择播放项目
        /// </summary>
        /// <param name="value"></param>
        void SelectPlayItem(string value)
        {
            try
            {
                this.Playlist.Select(value);
            }
            catch (Exception e)
            {
                LogUtil.Debug(e, "Player::SelectPlayItem:");
            }
        }

        /// <summary>
        /// 加载收藏项目
        /// </summary>
        void LoadFavorite()
        {
            StartCoroutine(this.Playlist.LoadFromFavorite());
        }

        /// <summary>
        /// 加载全部项目
        /// </summary>
        void LoadAll()
        {
            StartCoroutine(this.Playlist.LoadAll());
        }

        /// <summary>
        /// 创建UI
        /// </summary>
        public void CreateUI()
        {
            _DebugUIs = new GroupUI(this);
            _CameraSettingUIs = new GroupUI(this);

            // Left UIs
            _LoadUIs.Add(Utils.SetupTwinButton(this, Lang.Get("Load Playlist"), () => StopPlayAndRun(this.Playlist.BeginImport), Lang.Get("Import From Folder"), () => StopPlayAndRun(_MMDFolderController.ImportFromFolder), LeftSide));

            var buttons = Utils.SetupTwinButton(this, "✓" + Lang.Get("Save Settings"), () => StartCoroutine(SaveSettings(CurrentItem)), Lang.Get("Favorite"), FavoriteOrUnFavorite, LeftSide);

            var saveButton = buttons.labelLeft;
            saveButton.color = new Color(0f, 0.6f, 0.3f, 1f);
            //saveButton.fontStyle = FontStyle.Bold;

            _FavoriteLabel = buttons.labelRight;

            _MMDUIs.Add(buttons);

            InitTip("Player");

            // Right UIs
            // 编辑模式切换按钮，默认不显示，进入编辑模式后显示，点击此按钮退出编辑模式
            var testButton = CreateButton($"<< {Lang.Get("Exit Edit Mode")}", RightSide);
            testButton.button.onClick.AddListener(OutEditMode);
            testButton.height = 100;
            testButton.buttonColor = Color.gray;
            _EditUIs.Add(testButton);

            _LoadButtons.Add(Utils.SetupTwinButton(this, Lang.Get("Load Folder"), () => StopPlayAndRun(_MMDFolderController.LoadFolder), Lang.Get("Load File"), () => _MMDFolderController.LoadFile(this.Playlist.CurrentMMD), RightSide));

            _PlayUIs.Add(SetupStaticClassChooser<MMDPlayMode>("Play Mode", MMDPlayMode.Names, MMDPlayMode.GetName(MMDPlayMode.Default), RightSide, m =>
            {
                this.Playlist.PlayMode = MMDPlayMode.GetValue(m);
            }));

            _MMDUIs.Add(_UIPlayButton = Utils.SetupButton(this, Lang.Get("Play"), () => TogglePlaying(), RightSide));

            _UIPlayButton.height = 60f;

            _PlayUIs.Add(Utils.SetupTwinButton(this, Lang.Get("Prev"), this.Prev, Lang.Get("Next"), this.Next, RightSide));

            _PlaylistChooser = new JSONStorableStringChooser("playlist", noneStrings, noneString, Lang.Get("Playlist"));// Utils.SetupStringChooser(this, Lang.Get("Playlist"), noneStrings, RightSide);
            _PlaylistChooser.setCallbackFunction = SelectPlayItem;
            _PlaylistChooser.isStorable = false;
            _PlaylistChooser.isRestorable = false;
            CreateFilterablePopup(_PlaylistChooser, RightSide);

            _PlayUIs.Add(_PlaylistChooser);

            _LoadTwinButton = Utils.SetupTwinButton(this, Lang.Get("Load Favorite"), () => StopPlayAndRun(LoadFavorite), Lang.Get("Load All"), () => StopPlayAndRun(LoadAll), RightSide);
            _LoadUIs.Add(_LoadTwinButton);

            _PlayUIs.Add(Utils.SetupTwinButton(this, Lang.Get("Remove Current"), this.Playlist.RemovePlayItem, Lang.Get("Clear All"), () => StopPlayAndRun(() => this.StartCoroutine(this.Playlist.ClearList())), RightSide));
            _PlayUIs.Add(Utils.SetupButton(this, Lang.Get("Export Playlist"), this.Playlist.BeginExport, RightSide));

            _LoadTips = CreateTextField(new JSONStorableString("Load Tips",
              Lang.Get("<color=#000><size=24>Click the buttons above to prepare to start your Dance!</size></color>" +
               "\n" +
               "\n" +
               "<b>Load Playlist</b> - Import the playlist settings from a saved file." +
               "\n" +
               "\n" +
               "<b>Import From Folder</b> - Load multiple MMDs to the playlist. You should select the upper directory where the MMD files are stored. " +
               "\n" +
               "\n" +
               "<b>Load Folder</b> - Load an MMD into temporary memory and You can test or adjust its play settings.If it works well, you can add it to the playlist." +
               "\n" +
               "\n" +
               "<b>Load File(vmd/wav/mp3/ogg)</b> - Load a file to the current MMD." +
               "\n" +
               "\n" +
               "<b>Load Favorites</b> - Load your favorite MMDs to the current Playlist." +
               "\n" +
               "\n" +
               "<b>Load All</b> - Load All MMDs to the current Playlist." +
               "")),
               LeftSide);
            _LoadTips.backgroundColor = new Color(1f, 0.92f, 0.016f, 0f);
            _LoadTips.height = 1024f;

            // 音频播放进度
            RegisterFloat(_AudioPlayController.playProgress);
            _MMDUIs.Add(CreateSlider(_AudioPlayController.playProgress, LeftSide));

            // 音频开始时间
            RegisterFloat(_AudioPlayController.startTime);
            _MMDUIs.Add(CreateSlider(_AudioPlayController.startTime, LeftSide));

            // 音频结束时间
            RegisterFloat(_AudioPlayController.endTime);
            _MMDUIs.Add(CreateSlider(_AudioPlayController.endTime, LeftSide));

            // 音频选择
            RegisterStringChooser(_AudioPlayController.audioChooser);
            _MMDUIs.Add(CreateScrollablePopup(_AudioPlayController.audioChooser, LeftSide));

            // 镜头选择
            RegisterStringChooser(_CameraMotionController.cameraChooser);
            var cameraPopup = CreateScrollablePopup(_CameraMotionController.cameraChooser, LeftSide);
            cameraPopup.popupPanelHeight = 600f;
            _MMDUIs.Add(cameraPopup);

            //_MMDUIs.Add(CreateUISpacer(RightSide, 20f));

            // 创建镜头UI
            CreateCameraUI();

            //_PlayUIs.Add(Utils.SetupSpacer(this, 1f, RightSide));

            _PlaySpeedJSON = new JSONStorableFloat(Lang.Get("Play Speed"), 1f, 0f, 2f);

            _PlaySpeedJSON.setCallbackFunction = s =>
            {
                // Time.timeScale = s;
                _AudioPlayController.SetPlaySpeed(s);
            };

            var playSpeedSlider = CreateSlider(_PlaySpeedJSON, RightSide);

            playSpeedSlider.ConfigureQuickButtons(-0.01f, -0.10f, -0.25f, -0.50f, 0.01f, 0.10f, 0.25f, 0.5f);

            _PlayUIs.Add(_PlaySpeedJSON);

            //_MMDUIs.Add(CreateUISpacer(LeftSide));

            var versionStr = $"<color=#000><size=35><b>{PLUGIN_NAME} v{this.Version}</b></size></color>";

            var pluginInfo = "" +
                versionStr +
                "\n" +
                Lang.Get("The VMD base processing mechanism of this plugin is based on the technical achievements of mmd2timeline, We are looking into the possibility of merging the two plugins, stay tuned.") +
                Lang.Get("" +
                "\n\n" +
                "https://space.bilibili.com/1016760606" +
                //"\n" +
                //"https://www.patreon.com/noone77521" +
                "\n\n" +
                "<b>Follow the page get latest versions.</b>" +
                "\n\n" +
                "<size=24><b>IMPORTANT!!!</b></size>" +
                "\n" +
                "<size=24>The MMD Folder Must In VAM Path.If Your MMD Folder In Other Path, Use The CMD Command 'mklink' Link it to VAM Path.</size>" +
                "\n") +
                "\n" +
                "\n" +
                "<size=18>" +
                Lang.Get("Technical References Catalog:") +
                "\n" +
                //"https://github.com/feelfar/varManager" +
                //"\n" +
                //Lang.Get("MMD2Timeline is based on the LoadScene module in the open source project varManager on github.") +
                //"\n" +
                //"\n" +
                "https://hub.virtamate.com/threads/mmd2timeline.32274/" +
                "\n" +
                Lang.Get("MMD2Timeline's vmd reading, and core functions including but not limited to basic skeleton and motion management are based on mmd2timeline's technical achievements, and we have been explicitly authorized to develop subsequent versions based on the corresponding technical foundation.") +
                "\n" +
                "\n" +
                "https://github.com/x3bits/libmmd-for-unity" +
                "\n" +
                Lang.Get("mmd2timeline has been improved based on Libmmd-for-Unity, and so MMD2Timeline has inherited the technical relationship.") +
                "\n" +
                "\n" +
                "https://github.com/lfe999/VamFreeMMD" +
                "\n" +
                Lang.Get("MMD2Timeline's audio load Method from the open source project VamFreeMMD.") +
                "https://github.com/lfe999/VamFreeMMD/blob/master/src/FreeMMD.cs#L316" +
                "\n" +
                "\n" +
                "https://hub.virtamate.com/resources/macgruber-essentials.160/" +
                "\n" +
                Lang.Get("Some of MMD2Timeline's UI processing uses MacGruber_Utils from MacGruber Essentials, and we've made a few feature tweaks and added some functionality. " +
                "The Capturer module, which we haven't publicly released yet, uses the SuperShot module from that plugin, and we modified its operation rules " +
                "and added memory release handling to meet the needs of frame-by-frame recording.That Plugin available for free under \"CC BY-SA\" license.") +
                "\n" +
                "\n" +
                "https://hub.virtamate.com/resources/searchlight.14512/" +
                "\n" +
                Lang.Get("MMD2Timeline's camera focus feature is based on Searchlight improvements.Searchlight basically \"CC BY-NC\".") +
                "\n" +
                "\n" +
                "https://github.com/acidbubbles/vam-passenger" +
                "\n" +
                Lang.Get("MMD2Timeline's follow-the-camera feature is based on Passenger, Passenger complies with the MIT license.") +
                "\n" +
                "\n" +
                Lang.Get("MMD2Timeline is a free VAM plugin. \nMMD2Timeline code is licensed under the AGPL 3.0 license, except for the code referenced above, which is licensed under the original license. \nDo not use it for unauthorized commercial use or in closed source projects.") +
                "</size>";

            var pluginInfoText = Utils.SetupInfoText(this, pluginInfo, 286f - 35f, RightSide);

            _PluginInfo = pluginInfoText.dynamicText;
            _PluginInfo.UItext.alignment = TextAnchor.MiddleCenter;
            _DebugUIs.OtherElements.Add(_PluginInfo);

            var recoverButton = Utils.SetupButton(this, Lang.Get("Reset Motion Model\n(Fix motion problems with bone changes)"), () =>
            {
                StartCoroutine(ResetAllPersonMotion());
            }, RightSide);

            recoverButton.height = 93;
            recoverButton.buttonColor = new Color(1f, 0f, 0f, 0.5f);

            _MMDUIs.Add(recoverButton);

            _ShowDebugInfoJSON = new JSONStorableBool(Lang.Get("Show Debug Info"), false);
            _ShowDebugInfoJSON.setCallbackFunction = v =>
            {
                _DebugUIs.RefreshView(v);
            };
            var debugToggle = CreateToggle(_ShowDebugInfoJSON, RightSide);

            _DebugUIs.ToggleBool = _ShowDebugInfoJSON;

            _DebugInfo = new JSONStorableString("DebugInfo", "DEBUG");
            var debugField = CreateTextField(_DebugInfo, RightSide);
            debugField.height = 300;

            _DebugUIs.Elements.Add(debugField);
        }

        /// <summary>
        /// 停止播放并获取目标方法
        /// </summary>
        /// <param name="action"></param>
        /// <returns></returns>
        private void StopPlayAndRun(UnityAction action)
        {
            this.StopPlaying();
            action?.Invoke();
        }

        /// <summary>
        /// 显示提示条
        /// </summary>
        /// <param name="message"></param>
        /// <param name="height"></param>
        /// <param name="alignment"></param>
        /// <param name="action"></param>
        internal override void ShowTip(string message = null, float height = 620, float step = 0, float? total = null, Action action = null, string buttonText = null, TextAnchor alignment = TextAnchor.MiddleCenter)
        {
            base.ShowTip(message, height, step, total, action, buttonText, alignment);

            // 隐藏编辑UI
            ShowEditUIs(false);
            // 隐藏MMDUI
            ShowMMDUIs(false);
            // 隐藏播放UI
            ShowPlayUIs(false);
            // 隐藏加载UI
            ShowLoadUIs(false);
            // 隐藏调试UI
            ShowDebugUIs(false);
            // 隐藏加载文件和目录的按钮
            ShowLoadButtons(false);
            _LoadTips.gameObject.SetActive(false);
        }

        /// <summary>
        /// 显示初始UI
        /// </summary>
        internal void ShowInitLoadUI()
        {
            if (this.Playlist.MMDCount > 0)
            {
                this.ShowPlayUI();
            }
            else
            {
                // 隐藏提示UI
                HideTip();
                //// 隐藏导入UI
                //ShowImportUIs(false);
                // 隐藏编辑UI
                ShowEditUIs(false);
                // 隐藏MMDUI
                ShowMMDUIs(false);
                // 隐藏播放UI
                ShowPlayUIs(false);
                // 显示加载UI
                ShowLoadUIs(true);
                // 显示加载文件和目录的按钮
                ShowLoadButtons(true);
                // 显示加载提示
                _LoadTips.gameObject.SetActive(true);

                _PluginInfo.height = 1000f;
            }
        }

        /// <summary>
        /// 显示普通播放UI
        /// </summary>
        private void ShowPlayUI()
        {
            // 隐藏提示UI
            HideTip();
            // 隐藏编辑UI
            ShowEditUIs(false);
            // 显示MMDUI
            ShowMMDUIs(true);
            // 显示播放UI
            ShowPlayUIs(true);
            // 显示加载UI
            ShowLoadUIs(true);
            // 显示加载文件和目录的按钮
            ShowLoadButtons(true);
            _LoadTips.gameObject.SetActive(false);
            ShowDebugUIs(true);

            _PluginInfo.height = 286f - 35f + 65f;
        }

        /// <summary>
        /// 显示编辑UI
        /// </summary>
        private void ShowEditUI()
        {
            // 隐藏提示UI
            HideTip();
            // 显示编辑UI
            ShowEditUIs(true);
            // 显示MMDUI
            ShowMMDUIs(true);
            // 显示加载文件和目录的按钮
            ShowLoadButtons(true);
            // 隐藏播放UI
            ShowPlayUIs(false);
            // 隐藏加载UI
            ShowLoadUIs(false);

            _LoadTips.gameObject.SetActive(false);
            ShowDebugUIs(true);
            _PluginInfo.height = 745f - 35f + 70f;
        }

        /// <summary>
        /// 显示动作UI
        /// </summary>
        /// <param name="show"></param>
        void ShowMMDUIs(bool show)
        {
            // 显示所有MMDUI
            ShowUIElements(_MMDUIs, show);

            // 调整镜头UI的显示
            if (show)
            {
                // 根据选中状态更新UI
                _CameraSettingUIs.RefreshView();
                heelSettingsGroup.RefreshView();
            }
            else
            {
                // 隐藏UI
                _CameraSettingUIs.RefreshView(false);
                heelSettingsGroup.RefreshView(false);
            }

            // 调整人物相关的UI显示
            foreach (var item in _MMDPersonMotionSettingUIs)
            {
                var ui = item.Value;

                ShowUIElements(ui.OtherElements, show);

                if (show)
                {
                    // 根据选中状态更新UI
                    ui.RefreshView();
                }
                else
                {
                    // 隐藏UI
                    ui.RefreshView(false);
                }
            }
        }

        /// <summary>
        /// 显示播放UI
        /// </summary>
        /// <param name="show"></param>
        void ShowPlayUIs(bool show)
        {
            ShowUIElements(_PlayUIs, show);
        }

        /// <summary>
        /// 显示编辑UI
        /// </summary>
        /// <param name="show"></param>
        void ShowEditUIs(bool show)
        {
            ShowUIElements(_EditUIs, show);
        }

        /// <summary>
        /// 显示调试UI
        /// </summary>
        /// <param name="show"></param>
        void ShowDebugUIs(bool show)
        {
            ShowUIElement(_DebugUIs.ToggleBool, show);
            ShowUIElements(_DebugUIs.OtherElements, show);
            if (show)
            {
                _DebugUIs.RefreshView();
            }
            else
            {
                _DebugUIs.RefreshView(false);
            }
        }

        /// <summary>
        /// 显示加载UI
        /// </summary>
        /// <param name="show"></param>
        void ShowLoadUIs(bool show)
        {
            ShowUIElements(_LoadUIs, show);
        }

        /// <summary>
        /// 现在建在文件和目录的按钮
        /// </summary>
        /// <param name="show"></param>
        void ShowLoadButtons(bool show)
        {
            ShowUIElements(_LoadButtons, show);
        }

        /// <summary>
        /// 进入编辑模式
        /// </summary>
        void InEditMode()
        {
            isEditing = true;

            this.ShowEditUI();
        }

        /// <summary>
        /// 退出编辑模式
        /// </summary>
        void OutEditMode()
        {
            this.Pause();

            isEditing = false;

            this.ShowPlayUI();
            _MMDFolderController.Clear();
        }

        /// <summary>
        /// 人物动作设置UI
        /// </summary>
        Dictionary<int, GroupUI> _MMDPersonMotionSettingUIs = new Dictionary<int, GroupUI>();

        /// <summary>
        /// 移除人物动作UI
        /// </summary>
        /// <param name="controller"></param>
        /// <param name="person"></param>
        void RemovePersonMotionUI(PersonMotionController controller, Atom person)
        {
            var personId = person.GetInstanceID();

            if (controller != null)
            {
                foreach (var motionChooser in controller.MotionChoosers)
                {
                    DeregisterStringChooser(motionChooser);
                }

                // 动作播放进度
                DeregisterFloat(controller.playProgress);

                DeregisterFloat(controller.timeDelay);

                // 是否启用表情
                DeregisterBool(controller.enableFace);

                // 是否启用高跟鞋
                DeregisterBool(controller.enableHeel);

                DeregisterFloat(controller.holdRotationMaxForceAdjust);

                DeregisterFloat(controller.footJointDriveXTargetAdjust);

                DeregisterFloat(controller.toeJointDriveXTargetAdjust);

                DeregisterFloat(controller.positionX);
                DeregisterFloat(controller.positionY);
                DeregisterFloat(controller.positionZ);

                DeregisterFloat(controller.rotationX);
                DeregisterFloat(controller.rotationY);
                DeregisterFloat(controller.rotationZ);

                DeregisterFloat(controller.motionScale);
                DeregisterBool(controller.UseAllJointsSettingsJSON);
                DeregisterFloat(controller.AllJointsDamperPercentJSON);
                DeregisterFloat(controller.AllJointsSpringPercentJSON);
                DeregisterFloat(controller.AllJointsMaxVelocityJSON);
                DeregisterBool(controller.CloseLowerBonesJSON);

                GroupUI group;

                if (_MMDPersonMotionSettingUIs.TryGetValue(personId, out group))
                {
                    var uiList = new List<object>() { group.ToggleBool };

                    if (group.ChildGroups.Count > 0)
                    {
                        foreach (var childGroup in group.ChildGroups)
                        {
                            uiList = uiList.Concat(childGroup.Elements.Select(e => e as object)).ToList();
                        }
                    }

                    uiList = uiList.Concat(group.Elements.Select(e => e as object)).ToList();

                    // 移除所有设定UI
                    Utils.RemoveUIElements(this, uiList);

                    // 移除其他UI
                    Utils.RemoveUIElements(this, group.OtherElements);
                }
            }
        }

        /// <summary>
        /// 创建人物动作UI
        /// </summary>
        /// <param name="controller"></param>
        /// <param name="person"></param>
        void CreatePersonMotionUI(PersonMotionController controller, Atom person)
        {
            if (controller != null)
            {
                var personId = person.GetInstanceID();
                var allJointsSettingsGroup = new GroupUI(this);
                heelSettingsGroup = new GroupUI(this);

                if (!this._MMDPersonMotionSettingUIs.ContainsKey(personId))
                {
                    this._MMDPersonMotionSettingUIs[personId] = new GroupUI(this);
                    this._MMDPersonMotionSettingUIs[personId].ChildGroups.Add(allJointsSettingsGroup);
                    //this._MMDPersonMotionSettingUIs[personId].ChildGroups.Add(heelSettingsGroup);
                }

                foreach (var motionChooser in controller.MotionChoosers)
                {
                    RegisterStringChooser(motionChooser);
                    var popup = CreateScrollablePopup(motionChooser, LeftSide);
                    popup.popupPanelHeight = 600f;
                    this._MMDPersonMotionSettingUIs[personId].OtherElements.Add(popup);
                }

                #region 高跟设置UI
                // 是否启用高跟鞋
                RegisterBool(controller.enableHeel);
                _MMDUIs.Add(CreateToggle(controller.enableHeel, LeftSide));
                heelSettingsGroup.ToggleBool = controller.enableHeel;

                controller.enableHeel.setCallbackFunction = v =>
                {
                    heelSettingsGroup.RefreshView(v);
                    controller.UpdateEnableHighHeel();
                };

                RegisterFloat(controller.holdRotationMaxForceAdjust);
                heelSettingsGroup.Elements.Add(CreateSlider(controller.holdRotationMaxForceAdjust, LeftSide));

                RegisterFloat(controller.footJointDriveXTargetAdjust);
                heelSettingsGroup.Elements.Add(CreateSlider(controller.footJointDriveXTargetAdjust, LeftSide));

                RegisterFloat(controller.toeJointDriveXTargetAdjust);
                heelSettingsGroup.Elements.Add(CreateSlider(controller.toeJointDriveXTargetAdjust, LeftSide));
                #endregion

                var toggleJSON = new JSONStorableBool($"{person.uid}\\{Lang.Get("Show Motion Settings")}", false);

                toggleJSON.setCallbackFunction = v =>
                {
                    _MMDPersonMotionSettingUIs[personId]?.RefreshView(v);
                };

                _MMDUIs.Add(CreateToggle(toggleJSON, LeftSide));
                _MMDPersonMotionSettingUIs[personId].ToggleBool = toggleJSON;

                // 动作播放进度
                RegisterFloat(controller.playProgress);
                _MMDPersonMotionSettingUIs[personId].Elements.Add(CreateSlider(controller.playProgress, LeftSide));

                RegisterFloat(controller.timeDelay);
                _MMDPersonMotionSettingUIs[personId].Elements.Add(CreateSlider(controller.timeDelay, LeftSide));

                // 是否启用表情
                RegisterBool(controller.enableFace);
                _MMDPersonMotionSettingUIs[personId].Elements.Add(CreateToggle(controller.enableFace, LeftSide));
                #region 位置设置UI

                // 位置
                RegisterFloat(controller.positionX);
                var positionXSlider = CreateSlider(controller.positionX, LeftSide);
                positionXSlider.valueFormat = "F4";
                _MMDPersonMotionSettingUIs[personId].Elements.Add(positionXSlider);

                RegisterFloat(controller.positionY);
                var positionYSlider = CreateSlider(controller.positionY, LeftSide);
                positionYSlider.valueFormat = "F4";
                _MMDPersonMotionSettingUIs[personId].Elements.Add(positionYSlider);

                RegisterFloat(controller.positionZ);
                var positionZSlider = CreateSlider(controller.positionZ, LeftSide);
                positionZSlider.valueFormat = "F4";
                _MMDPersonMotionSettingUIs[personId].Elements.Add(positionZSlider);

                //角度
                RegisterFloat(controller.rotationX);
                var rotationXSlider = CreateSlider(controller.rotationX, LeftSide);
                _MMDPersonMotionSettingUIs[personId].Elements.Add(rotationXSlider);

                RegisterFloat(controller.rotationY);
                var rotationYSlider = CreateSlider(controller.rotationY, LeftSide);
                _MMDPersonMotionSettingUIs[personId].Elements.Add(rotationYSlider);

                RegisterFloat(controller.rotationZ);
                var rotationZSlider = CreateSlider(controller.rotationZ, LeftSide);
                _MMDPersonMotionSettingUIs[personId].Elements.Add(rotationZSlider);

                #endregion
                RegisterFloat(controller.motionScale);
                _MMDPersonMotionSettingUIs[personId].Elements.Add(CreateSlider(controller.motionScale, LeftSide));

                #region 全部关节设置
                RegisterBool(controller.UseAllJointsSettingsJSON);
                _MMDPersonMotionSettingUIs[personId].Elements.Add(CreateToggle(controller.UseAllJointsSettingsJSON, LeftSide));
                allJointsSettingsGroup.ToggleBool = controller.UseAllJointsSettingsJSON;

                controller.UseAllJointsSettingsJSON.setCallbackFunction = v =>
                {
                    allJointsSettingsGroup.RefreshView(v);

                    controller.SetPersonAllJoints();
                };

                RegisterFloat(controller.AllJointsSpringPercentJSON);
                var sliderSpringPercent = CreateSlider(controller.AllJointsSpringPercentJSON, LeftSide);
                sliderSpringPercent.valueFormat = "F4";
                allJointsSettingsGroup.Elements.Add(sliderSpringPercent);

                RegisterFloat(controller.AllJointsDamperPercentJSON);
                var sliderDamperPercent = CreateSlider(controller.AllJointsDamperPercentJSON, LeftSide);
                sliderDamperPercent.valueFormat = "F4";
                allJointsSettingsGroup.Elements.Add(sliderDamperPercent);

                RegisterFloat(controller.AllJointsMaxVelocityJSON);
                var sliderMaxVelocity = CreateSlider(controller.AllJointsMaxVelocityJSON, LeftSide);
                sliderMaxVelocity.valueFormat = "F4";
                allJointsSettingsGroup.Elements.Add(sliderMaxVelocity);
                #endregion

                // 关闭下半身骨骼
                RegisterBool(controller.CloseLowerBonesJSON);
                _MMDPersonMotionSettingUIs[personId].Elements.Add(CreateToggle(controller.CloseLowerBonesJSON, LeftSide));
            }
        }

        /// <summary>
        /// 初始化镜头设置UI
        /// </summary>
        void CreateCameraUI()
        {
            var toggleJSON = new JSONStorableBool(Lang.Get("Show Camera Settings"), false);

            toggleJSON.setCallbackFunction = v =>
            {
                _CameraSettingUIs?.RefreshView(v);
            };

            _MMDUIs.Add(CreateToggle(toggleJSON, RightSide));
            _CameraSettingUIs.ToggleBool = toggleJSON;

            // 镜头动作播放进度
            RegisterFloat(_CameraMotionController.playProgress);
            _CameraSettingUIs.Elements.Add(CreateSlider(_CameraMotionController.playProgress, RightSide));

            RegisterFloat(_CameraMotionController.timeDelay);
            _CameraSettingUIs.Elements.Add(CreateSlider(_CameraMotionController.timeDelay, RightSide));

            RegisterFloat(_CameraMotionController.PositionOffsetXJSON);
            var positionOffsetXUI = CreateSlider(_CameraMotionController.PositionOffsetXJSON, RightSide);
            positionOffsetXUI.valueFormat = "F4";
            _CameraSettingUIs.Elements.Add(positionOffsetXUI);

            RegisterFloat(_CameraMotionController.PositionOffsetYJSON);
            var positionOffsetYUI = CreateSlider(_CameraMotionController.PositionOffsetYJSON, RightSide);
            positionOffsetYUI.valueFormat = "F4";
            _CameraSettingUIs.Elements.Add(positionOffsetYUI);

            RegisterFloat(_CameraMotionController.PositionOffsetZJSON);
            var positionOffsetZUI = CreateSlider(_CameraMotionController.PositionOffsetZJSON, RightSide);
            positionOffsetZUI.valueFormat = "F4";
            _CameraSettingUIs.Elements.Add(positionOffsetZUI);

            RegisterFloat(_CameraMotionController.RotationOffsetXJSON);
            _CameraSettingUIs.Elements.Add(CreateSlider(_CameraMotionController.RotationOffsetXJSON, RightSide));

            RegisterFloat(_CameraMotionController.RotationOffsetYJSON);
            _CameraSettingUIs.Elements.Add(CreateSlider(_CameraMotionController.RotationOffsetYJSON, RightSide));

            RegisterFloat(_CameraMotionController.RotationOffsetZJSON);
            _CameraSettingUIs.Elements.Add(CreateSlider(_CameraMotionController.RotationOffsetZJSON, RightSide));

            RegisterFloat(_CameraMotionController.CameraScaleJSON);
            _CameraSettingUIs.Elements.Add(CreateSlider(_CameraMotionController.CameraScaleJSON, RightSide));
        }

        #endregion
    }
}
