using MacGruber;
using mmd2timeline.Player.Controllers;
using mmd2timeline.Player.Model;
using System;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;

namespace mmd2timeline.Player
{
    internal partial class Settings
    {
        /// <summary>
        /// 镜头聚焦UI清单
        /// </summary>
        GroupUI _CameraFocusUI;

        List<JSONStorableBool> _StorableBools = new List<JSONStorableBool>();
        List<JSONStorableFloat> _StorableFloats = new List<JSONStorableFloat>();
        List<JSONStorableStringChooser> _StorableStrings = new List<JSONStorableStringChooser>();

        /// <summary>
        /// 脚部放松UI清单
        /// </summary>
        List<object> _FreeFootUI = new List<object>();

        JSONStorableFloat springSlider;

        JSONStorableFloat damperSlider;

        Player _Player;

        /// <summary>
        /// 获取Player
        /// </summary>
        protected Player Player
        {
            get
            {
                if (_Player == null)
                {
                    _Player = MacGruber.Utils.FindWithinSamePlugin<Player>(this);

                    if (_Player == null)
                    {
                        LogUtil.Debug("Plugin 'Player' not found.");
                    }
                }
                return _Player;
            }
        }

        /// <summary>
        /// 初始化
        /// </summary>
        public override void Init()
        {
            base.Init();
            Utils.OnInitUI(this, CreateAllUIElement);
        }

        /// <summary>
        /// 首次调用Update之前调用的方法
        /// </summary>
        public virtual void Start()
        {
            LogUtil.Debug("Settings::START!!!");

            try
            {
                InitSettingUI();
            }
            catch (Exception ex)
            {
                LogUtil.LogError(ex, "GeneralSettings::Start:");
            }
            _CameraFocusUI.RefreshView();

            AudioPlayController.GetInstance().SetVolume(config.Volume);
        }

        /// <summary>
        /// 初始化设置UI
        /// </summary>
        protected void InitSettingUI()
        {
            InitGeneralSettingUI();
            InitPhysicsMeshUI();
            InitAutoCorrectUI();
            InitCameraSettingUI();
            InitShortcutKey();
            InitLanguageUI();

            Utils.SetupButton(this, Lang.Get("Reset All Settings To Default"), () =>
            {
                try
                {
                    config.ResetAll();

                    SetValuesToDefalut();
                }
                catch (Exception ex)
                {
                    LogUtil.LogError(ex, "InitSettingUI::Reset:");
                }
            }, RightSide);
        }

        private Atom _WindowCameraAtom;

        /// <summary>
        /// 获取WindowCamera原子
        /// </summary>
        protected Atom WindowCameraAtom
        {
            get
            {
                if (_WindowCameraAtom == null)
                {
                    _WindowCameraAtom = SuperController.singleton.GetAtoms().FirstOrDefault(a => a.type == "WindowCamera");
                }
                return _WindowCameraAtom;
            }
        }

        void InitShortcutKey()
        {
            CreateTitleUI("Shortcut Keys", RightSide);

            SetupToggle(config.EnableSpacePlay, "Enable Space to Toggle Play/Pause", dft.EnableSpacePlay, RightSide);
            SetupToggle(config.EnableRightArrow, "Enable Right Arrow to Forward 1s", dft.EnableRightArrow, RightSide);
            SetupToggle(config.EnableLeftArrow, "Enable Left Arrow to Back 1s", dft.EnableLeftArrow, RightSide);
            SetupToggle(config.EnableUpArrow, "Enable Up Arrow to Play Previous", dft.EnableUpArrow, RightSide);
            SetupToggle(config.EnableDownArrow, "Enable Down Arrow to Play Next", dft.EnableDownArrow, RightSide);

            Utils.SetupInfoOneLine(this, Lang.Get("U - Toggle UI Hidden/Visible"), RightSide);
            Utils.SetupInfoOneLine(this, Lang.Get("M - Toggle WindowCamera View"), RightSide);

            Utils.SetupSpacer(this, 10f, RightSide);
        }

        /// <summary>
        /// 初始化语言UI
        /// </summary>
        void InitLanguageUI()
        {
            CreateTitleUI("Language Settings", RightSide);

            Utils.SetupButton(this, Lang.Get("Generate Language Profile"), () =>
            {
                lang.GenerateProfile();
            }, RightSide);

            var textInfo = Utils.SetupInfoTextNoScroll(this, Lang.Get("If you need to set your own language, press the \"Generate Language Profile\" button, it will generate the Saves\\PluginData\\MMD2Timeline\\lang.json file, you can modify the content in it to get your own language profiles."), 240, RightSide);
            textInfo.text.alignment = TextAnchor.MiddleCenter;

            Utils.SetupSpacer(this, 10f, RightSide);
        }

        /// <summary>
        /// 初始化镜头设置UI
        /// </summary>
        void InitCameraSettingUI()
        {
            _CameraFocusUI = new GroupUI(this);

            CreateTitleUI("Camera Settings", RightSide);

            SetupToggle(config.CameraEnabled, "Camera Enabled Non-VR", dft.CameraEnabled, (v) => config.CameraEnabled = v, RightSide);

            SetupToggle(config.CameraEnabledInVR, "Camera Enabled in VR", dft.CameraEnabledInVR, (v) => config.CameraEnabledInVR = v, RightSide);

            SetupSliderFloat(config.CameraPositionSmoothing, "Camera Position Smoothing", dft.CameraPositionSmoothing, 0f, 1f, (v) => config.CameraPositionSmoothing = v, RightSide, "F4");

            SetupSliderFloat(config.CameraRotationSmoothing, "Camera Rotation Smoothing", dft.CameraRotationSmoothing, 0f, 1f, (v) => config.CameraRotationSmoothing = v, RightSide, "F4");

            SetupToggle(config.DeactiveCameraWhenMainHUDOpened, "Deactive when MainHUD Opened", dft.DeactiveCameraWhenMainHUDOpened, (v) => config.DeactiveCameraWhenMainHUDOpened = v, RightSide);

            SetupToggle(config.CameraOnlyKeyFrame, "Only Keyframes", dft.CameraOnlyKeyFrame, v => config.CameraOnlyKeyFrame = v, RightSide);

            var _CameraMotionController = CameraMotionController.GetInstance();

            #region 窗口镜头同步
            var syncWindowCameraJSON = SetupToggle(config.SyncWindowCamera, "Play in WindowCamera", dft.SyncWindowCamera, RightSide);//_WindowCameraAtom

            _StorableBools.Add(syncWindowCameraJSON);

            syncWindowCameraJSON.setCallbackFunction = v =>
            {
                if (v)
                {
                    config.SyncWindowCamera = v;
                }
                else
                {
                    config.SyncWindowCamera = false;

                    if (v)
                    {
                        syncWindowCameraJSON.valNoCallback = false;
                    }
                }
            };

            #endregion

            RegisterBool(_CameraMotionController.FocusOnAtomJSON);
            _CameraMotionController.FocusOnAtomJSON.setCallbackFunction = v => _CameraFocusUI.RefreshView(v);
            CreateToggle(_CameraMotionController.FocusOnAtomJSON, rightSide: true);
            _CameraFocusUI.ToggleBool = _CameraMotionController.FocusOnAtomJSON;

            RegisterStringChooser(_CameraMotionController.FocusAtomJSON);
            var dp = CreateScrollablePopup(_CameraMotionController.FocusAtomJSON, rightSide: true);
            dp.popupPanelHeight = 1100f;
            _CameraFocusUI.Elements.Add(dp);

            RegisterStringChooser(_CameraMotionController.FocusReceiverJSON);
            var fr = CreateScrollablePopup(_CameraMotionController.FocusReceiverJSON, rightSide: true);
            fr.popupPanelHeight = 1100f;
            _CameraFocusUI.Elements.Add(fr);

            Utils.SetupSpacer(this, 10f, RightSide);
        }

        /// <summary>
        /// 根据动作模式设置弹簧和阻尼的值
        /// </summary>
        /// <param name="v"></param>
        void SetValueByMode(int v)
        {
            LogUtil.Debug($"SetValueByMode::{v}");

            if (springSlider == null || damperSlider == null)
            {
                return;
            }

            if (config.MotionModes.Any(m => m.Mode == v))
            {
                var mode = config.MotionModes.FirstOrDefault(m => m.Mode == v);

                springSlider.val = mode.Spring;
                damperSlider.val = mode.Damper;
            }
        }

        /// <summary>
        /// 初始化通用设置UI
        /// </summary>
        void InitGeneralSettingUI()
        {
            //var fpsSlider = Utils.SetupSliderInt(this, "FPS", 60, 0, 300, LeftSide);
            //fpsSlider.setCallbackFunction = v => UnityEngine.Application.targetFrameRate = (int)v;

            CreateTitleUI("General Settings", LeftSide);

            SetupSliderFloat(config.Volume, "Play Volume", dft.Volume, 0f, 1f, v =>
            {
                config.Volume = v;
                AudioPlayController.GetInstance().SetVolume(config.Volume);
            }, LeftSide, "F4");

            SetupToggle(config.ResetPhysicalWhenLoadMotion, "Reset Person Physical When Motion Reset", dft.ResetPhysicalWhenLoadMotion, v => config.ResetPhysicalWhenLoadMotion = v, LeftSide);

            SetupToggle(config.LockPersonPosition, "Lock Person Position", dft.LockPersonPosition, v => config.LockPersonPosition = v, LeftSide);

            var motionModeChooser = SetupStaticClassChooser<MotionMode>("Motion Mode", MotionMode.Names, MotionMode.GetName(MotionMode.Normal), LeftSide, v => { SetValueByMode(MotionMode.GetValue(v)); });

            springSlider = SetupSliderFloat(config.AllJointsSpringPercent, "All Joints Spring Percent", dft.AllJointsSpringPercent, 0f, 1f, v =>
            {
                config.AllJointsSpringPercent = v;

                motionModeChooser.val = Lang.Get(config.CurrentMotionMode.ToString());

                Player.SetAllPersonJointsSpringPercent();
            }, LeftSide, "F4");

            damperSlider = SetupSliderFloat(config.AllJointsDamperPercent, "All Joints Damper Percent", dft.AllJointsDamperPercent, 0f, 1f, v =>
            {
                config.AllJointsDamperPercent = v;

                motionModeChooser.val = Lang.Get(config.CurrentMotionMode.ToString());

                Player.SetAllPersonJointsDamperPercent();
            }, LeftSide, "F4");

            SetupSliderFloat(config.AllJointsMaxVelocity, "All Joints Max Velocity", dft.AllJointsMaxVelocity, 0f, 100f, v =>
            {
                config.AllJointsMaxVelocity = v;
                Player.SetAllPersonJointsMaxVelocity();
            }, LeftSide, "F4");

            SetupStaticClassChooser<PositionState>(PositionState.GetName((int)config.MotionPositionState), "Motion Position State", PositionState.Names, PositionState.GetName((int)dft.MotionPositionState), LeftSide, m =>
            {
                config.MotionPositionState = (FreeControllerV3.PositionState)PositionState.GetValue(m);
                Player.ResetPersonControllerState();
            });

            SetupStaticClassChooser<RotationState>(RotationState.GetName((int)config.MotionRotationState), "Motion Rotation State", RotationState.Names, RotationState.GetName((int)dft.MotionRotationState), LeftSide, m =>
            {
                config.MotionRotationState = (FreeControllerV3.RotationState)RotationState.GetValue(m);
                Player.ResetPersonControllerState();
            });

            SetupSliderFloat(config.GlobalMotionScale, "Global Motion Scale", dft.GlobalMotionScale, 0f, 2f, v => config.GlobalMotionScale = v, LeftSide, "F4");

            Utils.SetupSpacer(this, 10f, LeftSide);
        }

        /// <summary>
        /// 初始化物理网格UI
        /// </summary>
        void InitPhysicsMeshUI()
        {
            CreateTitleUI("Physics Mesh Settings", RightSide);

            SetupToggle(config.MouthPhysicsMesh, "Enable Mouth Physics Mesh", dft.MouthPhysicsMesh, (v) =>
            {
                config.MouthPhysicsMesh = v;

                foreach (var item in Player.PersonMotionControllers)
                {
                    var controller = item.Value;
                    controller.SetPhysicsMesh("MouthPhysicsMesh", v);
                }
            }, RightSide);

            SetupToggle(config.BreastPhysicsMesh, "Enable Breast Physics Mesh", dft.BreastPhysicsMesh, (v) =>
            {
                config.BreastPhysicsMesh = v;

                foreach (var item in Player.PersonMotionControllers)
                {
                    var controller = item.Value;
                    controller.SetPhysicsMesh("BreastPhysicsMesh", v);
                }
            }, RightSide);

            SetupToggle(config.LowerPhysicsMesh, "Enable Lower Physics Mesh", dft.LowerPhysicsMesh, (v) =>
            {
                config.LowerPhysicsMesh = v;
                foreach (var item in Player.PersonMotionControllers)
                {
                    var controller = item.Value;
                    controller.SetPhysicsMesh("LowerPhysicsMesh", v);
                }
            }, RightSide);

            Utils.SetupSpacer(this, 10f, RightSide);
        }

        /// <summary>
        /// 初始化自动高度修正UI
        /// </summary>
        void InitAutoCorrectUI()
        {
            CreateTitleUI("Auto-Correct Height Settings", LeftSide);

            SetupStaticClassChooser<AutoCorrectHeightMode>(AutoCorrectHeightMode.GetName(config.AutoFixHeightMode), "Auto-Correct Mode", AutoCorrectHeightMode.Names, AutoCorrectHeightMode.GetName(dft.AutoFixHeightMode), LeftSide, m => config.AutoFixHeightMode = AutoCorrectHeightMode.GetValue(m));

            SetupSliderFloat(config.AutoCorrectFixHeight, "Fix Height", dft.AutoCorrectFixHeight, 0f, 1f, v => config.AutoCorrectFixHeight = v, LeftSide, "F4");

            SetupSliderFloat(config.AutoCorrectFloorHeight, "Floor Height", dft.AutoCorrectFloorHeight, -1f, 1f, v => config.AutoCorrectFloorHeight = v, LeftSide, "F4");

            SetupToggle(config.EnableFootFree, "Enable Foot Free", dft.EnableFootFree, (v) =>
            {
                config.EnableFootFree = v;

                try
                {
                    ShowUIElements(_FreeFootUI, v);
                }
                catch (Exception e)
                {
                    LogUtil.LogError(e, $"Enable Foot Free");
                }
            }, LeftSide);

            var enableFootOffJSON = SetupToggle(config.EnableFootOff, "Enable Foot Off", dft.EnableFootOff, (v) => config.EnableFootOff = v, LeftSide);
            _FreeFootUI.Add(enableFootOffJSON);

            var enableKneeOffJSON = SetupToggle(config.EnableKneeOff, "Enable Knee Off", dft.EnableKneeOff, (v) => config.EnableKneeOff = v, LeftSide);
            _FreeFootUI.Add(enableKneeOffJSON);

            _FreeFootUI.Add(SetupSliderFloat(config.ToeOffHeight, "Toe Off Height", dft.ToeOffHeight, 0f, 1f, v => config.ToeOffHeight = v, LeftSide, "F4"));

            _FreeFootUI.Add(SetupSliderFloat(config.FootFixHeight, "Foot Fix Height", dft.FootFixHeight, 0f, 1f, v => config.FootFixHeight = v, LeftSide, "F4"));

            _FreeFootUI.Add(SetupSliderFloat(config.FootOffHeight, "Foot Off Height", dft.FootOffHeight, 0f, 1f, v => config.FootOffHeight = v, LeftSide, "F4"));

            _FreeFootUI.Add(SetupSliderFloat(config.FreeFootJointDriveXTarget, "Free Foot Joint Drive X Target", dft.FreeFootJointDriveXTarget, -65f, 40f, v => config.FreeFootJointDriveXTarget = v, LeftSide));

            _FreeFootUI.Add(SetupSliderFloat(config.FreeFootHoldRotationMaxForce, "Free Foot Hold Rotation Max Force", dft.FreeFootHoldRotationMaxForce, 0f, 1000f, v => config.FreeFootHoldRotationMaxForce = v, LeftSide));

            _FreeFootUI.Add(SetupSliderFloat(config.FreeKneeHoldRotationMaxForce, "Free Knee Hold Rotation Max Force", dft.FreeKneeHoldRotationMaxForce, 0f, 1000f, v => config.FreeKneeHoldRotationMaxForce = v, LeftSide));

            _FreeFootUI.Add(SetupSliderFloat(config.FreeThighHoldRotationMaxForce, "Free Thigh Hold Rotation Max Force", dft.FreeThighHoldRotationMaxForce, 0f, 1000f, v => config.FreeThighHoldRotationMaxForce = v, LeftSide));

            try
            {
                ShowUIElements(_FreeFootUI, config.EnableFootFree);
            }
            catch (Exception e)
            {
                LogUtil.LogError(e, $"ShowUIElements");
            }

            Utils.SetupSpacer(this, 10f, LeftSide);
        }

        /// <summary>
        /// 设置到默认值
        /// </summary>
        private void SetValuesToDefalut()
        {
            foreach (var item in _StorableBools) { item.SetValToDefault(); }
            foreach (var item in _StorableFloats) { item.SetValToDefault(); }
            foreach (var item in _StorableStrings) { item.SetValToDefault(); }
        }

        /// <summary>
        /// 设置Slider
        /// </summary>
        /// <param name="label"></param>
        /// <param name="defaultValue"></param>
        /// <param name="minValue"></param>
        /// <param name="maxValue"></param>
        /// <param name="callback"></param>
        /// <param name="rightSide"></param>
        /// <param name="valueFormat"></param>
        private JSONStorableFloat SetupSliderFloat(float value, string label, float defaultValue, float minValue, float maxValue, Action<float> callback, bool rightSide, string valueFormat = "")
        {
            var slider = Utils.SetupSliderFloat(this, Lang.Get(label), defaultValue, minValue, maxValue, callback, rightSide, valueFormat);
            slider.val = value;

            _StorableFloats.Add(slider);

            return slider;
        }

        /// <summary>
        /// 设置Toggle
        /// </summary>
        /// <param name="label"></param>
        /// <param name="defaultValue"></param>
        /// <param name="callback"></param>
        /// <param name="rightSide"></param>
        private JSONStorableBool SetupToggle(bool value, string label, bool defaultValue, Action<bool> callback, bool rightSide)
        {
            var toggle = Utils.SetupToggle(this, Lang.Get(label), defaultValue, callback, rightSide);
            toggle.val = value;

            _StorableBools.Add(toggle);

            return toggle;
        }

        /// <summary>
        /// 设置Toggle
        /// </summary>
        /// <param name="label"></param>
        /// <param name="defaultValue"></param>
        /// <param name="rightSide"></param>
        /// <returns></returns>
        private JSONStorableBool SetupToggle(bool value, string label, bool defaultValue, bool rightSide)
        {
            var toggle = Utils.SetupToggle(this, Lang.Get(label), defaultValue, rightSide);
            toggle.val = value;

            _StorableBools.Add(toggle);

            return toggle;
        }

        ///// <summary>
        ///// 设置选择器
        ///// </summary>
        ///// <typeparam name="TEnum"></typeparam>
        ///// <param name="self"></param>
        ///// <param name="label"></param>
        ///// <param name="defaultValue"></param>
        ///// <param name="rightSide"></param>
        ///// <param name="callback"></param>
        //private JSONStorableStringChooser SetupEnumChooser<TEnum>(TEnum value, string label, TEnum defaultValue, bool rightSide, EnumSetCallback<TEnum> callback)
        //    where TEnum : struct, IComparable, IConvertible, IFormattable
        //{
        //    var chooser = SetupEnumChooser(label, defaultValue, rightSide, callback);
        //    chooser.val = Lang.Get(value.ToString());

        //    _StorableStrings.Add(chooser);
        //    return chooser;
        //}

        private JSONStorableStringChooser SetupStaticClassChooser<T>(string value, string label, List<string> names, string defaultValue, bool rightSide, StaticClassSetCallback<T> callback)
        {
            var chooser = SetupStaticClassChooser<T>(label, names, defaultValue, rightSide, callback);
            chooser.val = Lang.Get(value.ToString());

            _StorableStrings.Add(chooser);
            return chooser;
        }
    }
}
